import re
import sha
from cStringIO import StringIO
import os
import sys
import tempfile
import logging

from trac.core import *
from trac.wiki.api import IWikiMacroProvider
from trac.mimeview.api import IHTMLPreviewRenderer, MIME_MAP
from trac.web import IRequestHandler
from trac.util import escape
from trac.wiki.formatter import wiki_to_oneliner
from trac import mimeview
from trac.wiki import IWikiSyntaxProvider

__author__ = 'Reza Lotun'

tex_preamble = r"""
\documentclass{article}
\usepackage{amsmath}
\usepackage{amsthm}
\usepackage{amssymb}
\usepackage{bm}
\pagestyle{empty}
\begin{document}
"""

cacheDirectory = 'c:/temp'

class TracMathPlugin(Component):
    implements(IWikiMacroProvider, IHTMLPreviewRenderer, IRequestHandler, IWikiSyntaxProvider)


    # IWikiMacroProvider methods
    def get_macros(self):
        self.load_config()
        yield 'latex'

    def get_macro_description(self, name):
        if name == 'latex':
            return 'LaTeX'

    # WANT TO USE THIS!
    #def expand_macro(formatter, name, content):
    #    """ Called by formatter when rendering the parsed wiki text """
    #    pass

    def internal_render_macro(self, name, content):
        if not name == 'latex':
            return 'Unknown macro %s' % (name)

        key = sha.new(content).hexdigest()
        imgname = key + '.png'
        imgpath = os.path.join(cacheDirectory, imgname)

        if not os.path.exists(imgpath):
            fd, texfn = tempfile.mkstemp(suffix='.tex', prefix='tracmath')

            f = os.fdopen(fd, 'w+')

            f.write(tex_preamble)
            f.write(content)
            f.write('\end{document}')

            f.close()

            os.chdir(cacheDirectory)
            cmd = self.latex_cmd + '%s' % texfn
            pin, pout, perr = os.popen3(cmd)
            pin.close()
            out = pout.read()
            err = perr.read()

            if len(err) and len(out):
                return 'Unable to call: %s %s %s' % (cmd, out, err)

            tex_base = os.path.basename(texfn.replace('.tex', ''))
            tex_path = os.path.dirname(texfn)

            cmd = self.dvipng_cmd + " -T tight -x 1200 -z 9 -bg Transparent " \
                  + "-o %s %s" % (imgpath, tex_base)
            pin, pout, perr = os.popen3(cmd)
            pin.close()
            out = pout.read()
            err = perr.read()

        #if len(err) and len(out):
        #    return 'Unable to call: %s %s %s' % (cmd, out, err)
        return imgname
    
    def render_macro(self, req, name, content):
        imgname = self.internal_render_macro(name, content)
        return '<img src="%s/tracmath/%s" alt="%s" style="vertical-align: middle"/>' \
               % (req.base_url, imgname, content)
        
    # IHTMLPreviewRenderer methods
    def get_quality_ratio(self, mimetype):
        if mimetype in ('application/tracmath'):
            return 2
        return 0

    def render(self, req, mimetype, content, filename=None, url=None):
        text = hasattr(content, 'read') and content.read() or content
        return self.render_macro(req, name, text)

    # IRequestHandler methods
    def match_request(self, req):
        #return re.match(r'/tracmath(?:_trac)?(?:/.*)?$', req.path_info)
        return req.path_info.startswith('/tracmath')

    def process_request(self, req):
        pieces = [item for item in req.path_info.split('/tracmath') if item]

        if pieces:
            pieces = [item for item in pieces[0].split('/') if item]
            if pieces:
                name = pieces[0]
                img_path = os.path.join(cacheDirectory, name)
                return req.send_file(img_path,
                        mimeview.get_mimetype(img_path))
        return

    # IWikiSyntaxProvider methods
    def get_wiki_syntax(self):
        """Replace textual patterns representing latex formulas with the
        corresponding image."""
        def _replace(formatter, namespace, match):
            output1 = self.internal_render_macro('latex', match.group(0))
            src = self.env.href('tracmath', output1)
            return '<img src="%s" alt="%s" style="vertical-align: middle" />' % (
                escape(src), escape(match.group(0)))
        yield '\$[^ ].+? \${1,2}', _replace

    def get_link_resolvers(self):
        return []

    def show_err(self, msg):
        """Display msg in an error box, using Trac style."""
        buf = StringIO()
        buf.write('<div id="content" class="error"><div class="message"> \n\
                   <strong>TracMath macro processor has detected an error. Please fix the problem before continuing.</strong> \n\
                   <pre>%s</pre> \n\
                   </div></div>' % escape(msg))
        self.log.error(msg)
        return buf
    
    def load_config(self):
        """Load the tracmath trac.ini configuration."""
        if 'tracmath' not in self.config.sections():
            return 'The <b>tracmath</b> section was not found in the trac configuration file.'
        cacheDirectory = self.config.get('tracmath', 'cache_dir')
        if not cacheDirectory:
            cacheDirectory = env['TMP']
            
        self.latex_cmd = self.config.get('tracmath', 'latex_cmd')
        self.dvipng_cmd = self.config.get('tracmath', 'dvipng_cmd')

        #TODO: check correct values.
        return ''
    
